@@ -452,6 +452,21 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                                                       rep(0,n_new)))),TOLERANCE)
     expect_lt(sum(abs(tail(pred$random_effect_cov)-c(0.1294617, 0.1286434, 0.1289687,
                                                      rep(0.8227741,n_new)))),TOLERANCE)
+    
+    # # Training using BFGS (sometimes crashes)
+    # gp_model <- GPModel(group_data = group_data_train, likelihood = "bernoulli_probit")
+    # gp_model$set_optim_params(params=list(optimizer_cov="bfgs"))
+    # bst <- gpboost(data = X_train,
+    #                label = y_train,
+    #                gp_model = gp_model,
+    #                nrounds = 1,
+    #                learning_rate = 0.1,
+    #                max_depth = 6,
+    #                min_data_in_leaf = 5,
+    #                objective = "binary",
+    #                verbose = 0)
+    # expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-c(0.3806467, 0.2682585))),TOLERANCE)
+  
   })
   
   test_that("GPBoost algorithm for binary classification when having only one grouping variable", {
@@ -659,6 +674,20 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     expect_lt(sum(abs(tail(pred$random_effect_cov)-c(rep(0.2166956 ,3),
                                                      rep(0.9927358 ,n_new)))),TOLERANCE)
     
+    # Training using BFGS
+    gp_model <- GPModel(group_data = group_data_train, likelihood = "bernoulli_probit")
+    gp_model$set_optim_params(params=list(optimizer_cov="bfgs"))
+    bst <- gpboost(data = X_train,
+                   label = y_train,
+                   gp_model = gp_model,
+                   nrounds = 30,
+                   learning_rate = 0.1,
+                   max_depth = 6,
+                   min_data_in_leaf = 5,
+                   objective = "binary",
+                   verbose = 0)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-0.9855134)),TOLERANCE)
+    
   })
   
   # This is a slow test
